﻿using DataSharing.Application.Identity;
using DataSharing.FwDataExchange;
using DataSharing.Province;
using DataSharing.RawData;
using DataSharing.SendTask;
using DataSharing.Share.Dtos.Province;
using DotNetCore.CAP;
using MapsterMapper;
using MediatR;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using XF.Domain.Repository;

namespace DataSharing.Host.Controllers
{
    public class TestController : BaseController
    {
        #region 注入
        private readonly IMapper _mapper;
        private readonly IMediator _mediator;
        private readonly ICapPublisher _capPublisher;
        private readonly IChannelConfigurationManager _channelConfigurationManager;
        private readonly IRepository<DsOrder> _dataOrderRepository;
        private readonly IRepository<DsReceiveProvinceData> _dsReceiveProvinceDataRepository;
        private readonly IRepository<DsGetCaseMaterialInfo> _getCaseMaterialInfoRepository;
        private readonly IRepository<DsGetCaseBackApply> _getCaseBackApplyRepository;
        private readonly IRepository<DsSendCaseInfo> _sendCaseInfoRepository;
        private readonly XieTongClient _xieTongClient;
        private readonly HuiJuClient _huiJuClient;
        private readonly FwClient _fwClient;
        private readonly IRepository<SendFwErrorData> _sendFwErrorDataRepository;

        private readonly IRepository<DsSendTask> _taskRepository;
        private readonly IIdentityAppService _identityAppService;
        private readonly IRepository<DsSendTaskInfo> _taskInfoRepository;
        private readonly IPusherProviderService _pusherProviderService;
        private readonly IRepository<DsTelCall> _dsTelCallRepository;
        private readonly ISharingConfigurationManager _sharingConfigurationManager;

        /// <summary>
        /// 
        /// </summary>
        /// <param name="mapper"></param>
        /// <param name="mediator"></param>
        /// <param name="capPublisher"></param>
        /// <param name="channelConfigurationManager"></param>
        /// <param name="dataOrderRepository"></param>
        /// <param name="dsReceiveProvinceDataRepository"></param>
        /// <param name="getCaseMaterialInfoRepository"></param>
        /// <param name="getCaseBackApplyRepository"></param>
        /// <param name="sendCaseInfoRepository"></param>
        /// <param name="xieTongClient"></param>
        /// <param name="fwClient"></param>
        /// <param name="sendFwErrorDataRepository"></param>
        public TestController(IMapper mapper,
             IMediator mediator,
             ICapPublisher capPublisher,
             IChannelConfigurationManager channelConfigurationManager,
             IRepository<DsOrder> dataOrderRepository,
             IRepository<DsReceiveProvinceData> dsReceiveProvinceDataRepository,
             IRepository<DsGetCaseMaterialInfo> getCaseMaterialInfoRepository,
             IRepository<DsGetCaseBackApply> getCaseBackApplyRepository,
             IRepository<DsSendCaseInfo> sendCaseInfoRepository,
             XieTongClient xieTongClient,
             FwClient fwClient,
             IRepository<SendFwErrorData> sendFwErrorDataRepository,

             HuiJuClient huiJuClient,
              IRepository<DsSendTask> taskRepository,
             IIdentityAppService identityAppService,
              IRepository<DsSendTaskInfo> taskInfoRepository,
              IPusherProviderService pusherProviderService,
              IRepository<DsTelCall> dsTelCallRepository,
              ISharingConfigurationManager sharingConfigurationManager)
        {
            _mapper = mapper;
            _mediator = mediator;
            _capPublisher = capPublisher;
            _channelConfigurationManager = channelConfigurationManager;
            _dataOrderRepository = dataOrderRepository;
            _dsReceiveProvinceDataRepository = dsReceiveProvinceDataRepository;
            _getCaseMaterialInfoRepository = getCaseMaterialInfoRepository;
            _getCaseBackApplyRepository = getCaseBackApplyRepository;
            _sendCaseInfoRepository = sendCaseInfoRepository;
            _xieTongClient = xieTongClient;
            _fwClient = fwClient;
            _sendFwErrorDataRepository = sendFwErrorDataRepository;

            _huiJuClient = huiJuClient;
            _taskRepository = taskRepository;
            _identityAppService = identityAppService;
            _taskInfoRepository = taskInfoRepository;
            _pusherProviderService = pusherProviderService;
            _dsTelCallRepository = dsTelCallRepository;
            _sharingConfigurationManager = sharingConfigurationManager;
        }
        #endregion

        /// <summary>
        /// 测试token
        /// </summary>
        /// <returns></returns>
        [HttpGet("gettesttokeninfo")]
        [AllowAnonymous]
        public async Task<List<TokenInfo>> GetToken()
        {
            //获取附件上传Token
            // string strToken = await _xieTongClient.GetTokenAsync(HttpContext.RequestAborted);
            //  var token = await _huiJuClient.GetTokenHJAsync(HttpContext.RequestAborted);

            var peizhi = _channelConfigurationManager.GetConfigurationProvince();
            var peizhi1 = _channelConfigurationManager.GetConfigurationBusinessFile();
            var peizhi2 = _channelConfigurationManager.GetConfigurationHotlineClient();

            var peizhi3 = _sharingConfigurationManager.IsCity(ConfigurationConsts.YiBin);
            var peizhi4 = _sharingConfigurationManager.GetCityCode();
            var peizhi5 = _sharingConfigurationManager.GetCityName();
            var peizhi6 = _sharingConfigurationManager.GetYiBinConfig();
            var peizhi7 = _sharingConfigurationManager.GetZiGongConfig();


            List<TokenInfo> tokenInfos = new()
            {
                await _huiJuClient.GetTokenAsync(peizhi.HuiJu, HttpContext.RequestAborted),
                await _huiJuClient.GetTokenAsync(peizhi.HuiJu, HttpContext.RequestAborted)
            };

            return tokenInfos;

            //  return token;
        }



        [HttpPost("test")]
        [AllowAnonymous]
        public async Task<string> Test([FromBody] LoginDto dto)
        {
            // return await _identityAppService.LoginAsync(dto, HttpContext.RequestAborted);

            var list = await _dsTelCallRepository.Queryable().Where(p => p.OverTime == null).ToListAsync();
            foreach (var item in list)
            {
                if (item.EndRingTimg != null)
                    item.AnsweredTime = item.EndRingTimg.Value.AddSeconds(2);
                else
                    item.AnsweredTime = item.BeginIvrTime.Value.AddSeconds(2);

                item.OverTime = item.AnsweredTime.Value.AddSeconds(item.Duration);

                Random random = new Random();

                // 生成一个从0到100的随机整数
                int randomNumber = random.Next(20, 180);
                item.FirstSendProvinceTime = item.OverTime.Value.AddSeconds(randomNumber);
                item.LastSendProvinceTime = item.FirstSendProvinceTime;
                await _dsTelCallRepository.UpdateAsync(item, HttpContext.RequestAborted);
            }

            var listOrder = await _dataOrderRepository.Queryable().Where(p => p.FirstSendProvinceTime == null).ToListAsync();
            foreach (var item in listOrder)
            {
                Random random = new Random();

                // 生成一个从0到100的随机整数
                int randomNumber = random.Next(20, 180);

                if (item.StartTime != null)
                    item.FirstSendProvinceTime = item.StartTime.Value.AddSeconds(randomNumber);
                else
                    item.FirstSendProvinceTime = item.CaseDate.Value.AddSeconds(randomNumber);

                item.LastSendProvinceTime = item.FirstSendProvinceTime;

                await _dataOrderRepository.UpdateAsync(item, HttpContext.RequestAborted);
            }

            return "成功";
        }

        [HttpGet("test1")]
        public async Task<string> Test1()
        {
            return DateTime.Now.ToString();
        }

        [HttpGet("test2")]
        [AllowAnonymous]
        public async Task<string> Test2()
        {
            return $"test - {DateTime.Now.ToString()}";
        }
    }
}
