﻿using DataSharing.Share.Dtos;
using Microsoft.Extensions.Logging;
using System.Net;
using DataSharing.SendTask;
using XF.Domain.Dependency;
using XF.Domain.Repository;
using DataSharing.Share.Dtos.YiBin.Enterprise;

namespace DataSharing.YiBin.Enterprise
{
    public class EnterprisePusherProviderService : IEnterprisePusherProviderService, IScopeDependency
    {
        private readonly ILogger<EnterprisePusherProviderService> _logger;
        private readonly IChannelConfigurationManager _channelConfigurationManager;
        private readonly IHttpClientFactory _httpClientFactory;
        private readonly EnterpriseClient _enterpriseClient;
        private readonly IRepository<DsSendTask> _dsSendTaskRepository;
        private readonly IRepository<DsSendTaskInfo> _dsSendTaskInfoRepository;
        private readonly ISharingConfigurationManager _sharingConfigurationManager;

        /// <summary>
        /// 
        /// </summary>
        /// <param name="logger"></param>
        /// <param name="channelConfigurationManager"></param>
        /// <param name="httpClientFactory"></param>
        /// <param name="enterpriseClient"></param>
        /// <param name="dsSendTaskRepository"></param>
        /// <param name="dsSendTaskInfoRepository"></param>
        /// <param name="sharingConfigurationManager"></param>
        public EnterprisePusherProviderService(ILogger<EnterprisePusherProviderService> logger,
            IChannelConfigurationManager channelConfigurationManager,
            IHttpClientFactory httpClientFactory,
           EnterpriseClient enterpriseClient,
            IRepository<DsSendTask> dsSendTaskRepository,
            IRepository<DsSendTaskInfo> dsSendTaskInfoRepository,
            ISharingConfigurationManager sharingConfigurationManager)
        {
            _logger = logger;
            _channelConfigurationManager = channelConfigurationManager;
            _httpClientFactory = httpClientFactory;
            _enterpriseClient = enterpriseClient;
            _dsSendTaskRepository = dsSendTaskRepository;
            _dsSendTaskInfoRepository = dsSendTaskInfoRepository;
            _sharingConfigurationManager = sharingConfigurationManager;
        }

        /// <summary>
        /// 企业服务推送数据
        /// </summary>
        /// <param name="dto"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        public async Task SendEnterpriseDataPusher(DsSendTask dto, CancellationToken cancellationToken)
        {
            if (dto is null)
                return;

            var request = dto.Request;

            //如果存在附件，需要将附件拉取下来加载到数据里面
            if (dto.FileJson != null && dto.FileJson.Count > 0)
            {
                var data = System.Text.Json.JsonSerializer.Deserialize<PushWorkOrder>(dto.Request);

                #region 根据附件Id拉取附件文件流
                //附件处理
                List<ReplyFiles> replyFiles = new();
                foreach (var item in dto.FileJson)
                {

                    var businessFile = _channelConfigurationManager.GetConfigurationBusinessFile();
                    byte[] fileContentBytes = null;
                    try
                    {
                        #region 读取数据
                        var baseAddressFile = businessFile.BaseUrl;
                        if (!baseAddressFile.EndsWith('/'))
                            baseAddressFile += "/";

                        string url = string.Format("{0}{1}?Source={2}&Id={3}", baseAddressFile, businessFile.DownloadUrlAddress, businessFile.Source, item.FileId);
                        using (var client = _httpClientFactory.CreateClient())
                        using (var response = await client.GetAsync(url))
                        {
                            if (response.StatusCode == HttpStatusCode.OK)
                            {
                                using var memoryStream = new MemoryStream();
                                await response.Content.CopyToAsync(memoryStream);
                                memoryStream.Seek(0, SeekOrigin.Begin);
                                fileContentBytes = memoryStream.ToArray();
                            }
                        }
                        #endregion

                        #region 组装附件参数
                        //组装附件参数
                        if (fileContentBytes != null && fileContentBytes.Length > 0)
                        {
                            ReplyFiles files = new ReplyFiles
                            {
                                Filename = item.FileId,
                                Fileformat = "",
                                FileText = Convert.ToBase64String(fileContentBytes)
                            };
                            //拉取文件流
                            replyFiles.Add(files);
                        }
                        #endregion
                    }
                    catch (Exception)
                    {

                    }
                }
                #endregion

                data.ReplyFiles = replyFiles;
                request = System.Text.Json.JsonSerializer.Serialize(data);
            }

            string baseAddress = _sharingConfigurationManager.GetYiBinConfig().Enterprise.AddressUrl;
            if (!baseAddress.EndsWith('/'))
                baseAddress += "/";

            var enterprise = await _enterpriseClient.RequestAsync<EnterpriseResponse>(baseAddress + dto.Path, dto.HttpMethod, request, cancellationToken);
            //写入推送明细表
            DsSendTaskInfo dsSendTaskInfo = new() { TaskId = dto.Id, RequestInfo = System.Text.Json.JsonSerializer.Serialize(enterprise) };
            //如果推送成功修改数据状态
            if (enterprise != null)
            {
                //看看需不需要处理一些信息
                dto.LastTime = DateTime.Now;
                dto.SendTimes = dto.SendTimes++;

                if (dto.FirstTime is null)
                    dto.FirstTime = DateTime.Now;

                if (enterprise.code == 200 && enterprise.success)
                {
                    dto.IsSuccess = true;
                    dsSendTaskInfo.IsSuccess = true;
                }
                await _dsSendTaskRepository.UpdateAsync(dto, cancellationToken);
            }

            await _dsSendTaskInfoRepository.AddAsync(dsSendTaskInfo, cancellationToken);
        }
    }
}
