﻿using Microsoft.Extensions.Options;
using XF.Domain.Dependency;

namespace DataSharing
{
    /// <summary>
    /// 获取系统配置
    /// </summary>
    public class ChannelConfigurationManager : IChannelConfigurationManager, IScopeDependency
    {
        private readonly IOptionsSnapshot<ChannelConfiguration> _channelOption;

        /// <summary>
        /// 
        /// </summary>
        /// <param name="channelOption"></param>
        public ChannelConfigurationManager(IOptionsSnapshot<ChannelConfiguration> channelOption)
        {
            _channelOption = channelOption;
        }

        /// <summary>
        /// 业务系统附件上传配置
        /// </summary>
        /// <returns></returns>
        public ConfigurationBusinessFile GetConfigurationBusinessFile()
        {
            return _channelOption.Value.BusinessFile;
        }

        /// <summary>
        /// 写入工单数据
        /// </summary>
        /// <returns></returns>
        public ConfigurationHotlineClient GetConfigurationHotlineClient()
        {
            return _channelOption.Value.HotlineClient;
        }

        /// <summary>
        /// 省12345配置
        /// </summary>
        /// <returns></returns>
        public ConfigurationProvince GetConfigurationProvince()
        {
            return _channelOption.Value.Province;
        }
    }



    /// <summary>
    /// 获取地市系统配置
    /// </summary>
    public interface ISharingConfigurationManager
    {
        bool IsCity(string cityCode);

        string GetCityCode();

        string GetCityName();

        CityYiBin GetYiBinConfig();

        CityZiGong GetZiGongConfig();
    }

    /// <summary>
    /// 获取地市系统配置
    /// </summary>
    public class SharingConfigurationManager : ISharingConfigurationManager, IScopeDependency
    {
        private readonly IOptionsSnapshot<SharingConfiguration> _sharingOption;

        public SharingConfigurationManager(IOptionsSnapshot<SharingConfiguration> sharingOption)
        {
            _sharingOption = sharingOption;
        }

        public bool IsCity(string cityCode) =>
            _sharingOption.Value.CityCode.Equals(cityCode, StringComparison.OrdinalIgnoreCase);

        public string GetCityCode() => _sharingOption.Value.CityCode;
        public string GetCityName() => _sharingOption.Value.CityName;

        public CityYiBin GetYiBinConfig() => _sharingOption.Value.YiBin;

        public CityZiGong GetZiGongConfig() => _sharingOption.Value.ZiGong;
    }
}
