﻿using Microsoft.AspNetCore.Http;
using Microsoft.Extensions.DependencyInjection;
using XF.Domain.Dependency;

namespace DataSharing.DataExchange
{
    public class DataExchangeClient : ISingletonDependency, ISelfDependency
    {
        private readonly IServiceScopeFactory _scopeFactory;
        private readonly IHttpClientFactory _httpClientFactory;

        /// <summary>
        /// 
        /// </summary>
        /// <param name="scopeFactory"></param>
        /// <param name="httpClientFactory"></param>
        public DataExchangeClient(IServiceScopeFactory scopeFactory, IHttpClientFactory httpClientFactory)
        {
            _scopeFactory = scopeFactory;
            _httpClientFactory = httpClientFactory;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <typeparam name="TResponse"></typeparam>
        /// <param name="path"></param>
        /// <param name="httpMethod"></param>
        /// <param name="request"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        public async Task<TResponse> ExecuteAsync<TResponse>(string path, string httpMethod, string request, CancellationToken cancellationToken = default)
        {
            var rsp = string.Empty;
            var client = _httpClientFactory.CreateClient();
            if (HttpMethods.IsGet(httpMethod))
            {
                rsp = await client.GetStringAsync(path, cancellationToken);
            }
            else
            if (HttpMethods.IsPost(httpMethod))
            {
                var dicParam = System.Text.Json.JsonSerializer.Deserialize<Dictionary<string, string>>(request);
                HttpContent content = new FormUrlEncodedContent(dicParam);
                var response = await client.PostAsync(path, content);
                response.EnsureSuccessStatusCode();
                using var responseContent = response.Content;
                rsp = await responseContent.ReadAsStringAsync(cancellationToken);

            }
            return System.Text.Json.JsonSerializer.Deserialize<TResponse>(rsp);
        }

    }
}
