﻿using DataSharing.RawData;
using DataSharing.SendTask;
using DataSharing.Share.Dtos.DataExchange;
using DotNetCore.CAP;
using Microsoft.Extensions.Logging;
using XF.Domain.Dependency;
using XF.Domain.Repository;

namespace DataSharing.DataExchange
{
    public class DataExchangePusherProviderService : IDataExchangePusherProviderService, IScopeDependency
    {
        private readonly ILogger<DataExchangePusherProviderService> _logger;
        private readonly DataExchangeClient _dataExchangeClient;
        private readonly IRepository<DsSendTask> _dsSendTaskRepository;
        private readonly ICapPublisher _capPublisher;
        private readonly IRepository<DsSendTaskInfo> _dsSendTaskInfoRepository;
        private readonly IRepository<DsUserTokenInfo> _dsUserTokenInfoRepository;

        /// <summary>
        /// 
        /// </summary>
        /// <param name="logger"></param>
        /// <param name="dataExchangeClient"></param>
        /// <param name="dsSendTaskRepository"></param>
        /// <param name="capPublisher"></param>
        /// <param name="dsSendTaskInfoRepository"></param>
        /// <param name="dsUserTokenInfoRepository"></param>
        public DataExchangePusherProviderService(ILogger<DataExchangePusherProviderService> logger,
             DataExchangeClient dataExchangeClient,
            IRepository<DsSendTask> dsSendTaskRepository,
            ICapPublisher capPublisher,
            IRepository<DsSendTaskInfo> dsSendTaskInfoRepository,
            IRepository<DsUserTokenInfo> dsUserTokenInfoRepository)
        {
            _logger = logger;
            _dataExchangeClient = dataExchangeClient;
            _dsSendTaskRepository = dsSendTaskRepository;
            _capPublisher = capPublisher;
            _dsSendTaskInfoRepository = dsSendTaskInfoRepository;
            _dsUserTokenInfoRepository = dsUserTokenInfoRepository;
        }

        /// <summary>
        /// 市州数据交换推送数据
        /// </summary>
        /// <param name="dto"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        public async Task SendDataExchangeDataPusher(DsSendTask dto, CancellationToken cancellationToken)
        {
            if (dto is null)
                return;
            string TaskType = dto.TaskType;

            var userInfo = await _dsUserTokenInfoRepository.GetAsync(p => p.PlatformSource == dto.TaskType, cancellationToken);
            if (userInfo is null || string.IsNullOrEmpty(userInfo.AddressUrl))
                return;

            var baseAddress = userInfo.AddressUrl;
            if (!baseAddress.EndsWith('/'))
                baseAddress += "/";

            var response = await _dataExchangeClient.ExecuteAsync<DataExchangeResponseDto>(baseAddress + dto.Path, dto.HttpMethod, dto.Request, cancellationToken);
            //写入推送明细表
            DsSendTaskInfo dsSendTaskInfo = new() { TaskId = dto.Id, RequestInfo = System.Text.Json.JsonSerializer.Serialize(response) };
            //如果推送成功修改数据状态
            if (response != null)
            {
                //看看需不需要处理一些信息
                dto.LastTime = DateTime.Now;
                dto.SendTimes = dto.SendTimes++;

                if (dto.FirstTime is null)
                    dto.FirstTime = DateTime.Now;

                if (response.Code == "1")
                {
                    dto.IsSuccess = true;
                    dsSendTaskInfo.IsSuccess = true;
                }

                CityTranspondRawDataDto cityRawDataDto = new()
                {
                    OrderCode = dto.ProvinceNo,
                    SendTimes = dto.SendTimes,
                    IsSuccess = dto.IsSuccess,
                    Result = Newtonsoft.Json.JsonConvert.SerializeObject(response),
                    NewCode = response.AcceptCode
                };
                await _capPublisher.PublishAsync("sharing.order.transpond.city", cityRawDataDto, cancellationToken: cancellationToken);

                await _dsSendTaskRepository.UpdateAsync(dto, cancellationToken);
            }
            await _dsSendTaskInfoRepository.AddAsync(dsSendTaskInfo, cancellationToken);
        }
    }
}
