﻿using Microsoft.Extensions.Logging;
using Quartz;
using XF.Domain.Queues;
using XF.Domain.Repository;

namespace DataSharing.SendTask;

public class TaskLoadJob : IJob, IDisposable
{
    private readonly IRepository<DsSendTask> _taskRepository;
    private readonly IQueue _queue;
    private readonly ILogger<TaskLoadJob> _logger;

    public TaskLoadJob(
        IRepository<DsSendTask> taskRepository,
        IQueue queue,
        ILogger<TaskLoadJob> logger)
    {
        _taskRepository = taskRepository;
        _queue = queue;
        _logger = logger;
    }

    public async Task Execute(IJobExecutionContext context)
    {
        var tasks = await _taskRepository.Queryable()
            .Where(d => !d.IsSuccess && d.SendTimes <= 6)
            .OrderByDescending(d => d.TaskPriority)
            .Take(10)
            .ToListAsync(context.CancellationToken);

        if (tasks.Any())
        {
            foreach (var sendTask in tasks)
            {
                await _queue.EnqueueConcurrentAsync(SendTaskDefaults.SendTaskQueue, sendTask, context.CancellationToken);
            }
        }

    }

    public void Dispose()
    {

    }
}